// Include GLFW
#include <GLFW/glfw3.h>
extern GLFWwindow* window; 

// Include GLM
#include <glm/glm.hpp>
#include <glm/gtc/matrix_transform.hpp>
using namespace glm;
#include <iostream>
#include "controls.hpp"

glm::mat4 ViewMatrix;
glm::mat4 ProjectionMatrix;

glm::mat4 getViewMatrix(){
	return ViewMatrix;
}
glm::mat4 getProjectionMatrix(){
	return ProjectionMatrix;
}


// Initial position : on +Z
glm::vec3 position = glm::vec3( -8, 4, 2 ); 
// Initial horizontal angle : toward -Z
float horizontalAngle = 3.14f;
// Initial vertical angle : none
float verticalAngle = -0.3f;
// Initial Field of View
float initialFoV = 45.0f;

float speed = 3.0f; // 3 units / second
float mouseSpeed = 0.005f;



void computeMatricesFromInputs(){

	// glfwGetTime is called only once, the first time this function is called
	static double lastTime = glfwGetTime();
	static double startTime = glfwGetTime();
	// Compute time difference between current and last frame
	double currentTime = glfwGetTime();
	double elapsedTime = double(currentTime - startTime);
	float deltaTime = float(currentTime - lastTime);

	// Get mouse position
	double xpos, ypos;
	glfwGetCursorPos(window, &xpos, &ypos);

	// Reset mouse position for next frame
	glfwSetCursorPos(window, 1024/2, 768/2);

	// Compute new orientation
	horizontalAngle += mouseSpeed * float(1024/2 - xpos );
	verticalAngle   += mouseSpeed * float( 768/2 - ypos );

	// Direction : Spherical coordinates to Cartesian coordinates conversion
	glm::vec3 direction(
		cos(verticalAngle) * sin(horizontalAngle), 
		sin(verticalAngle),
		cos(verticalAngle) * cos(horizontalAngle)
	);
	
	// Right vector
	glm::vec3 right = glm::vec3(
		sin(horizontalAngle - 3.14f/2.0f), 
		0,
		cos(horizontalAngle - 3.14f/2.0f)
	);
	
	// Up vector
	glm::vec3 up = glm::cross( right, direction );
	glm::vec3 look;
	// Start

	if (elapsedTime < 5.5) {
		// Go forward
		position += glm::vec3(1, 0, 0) * deltaTime* speed;
		
		// Return the camera 
		if (elapsedTime > 3.5 && elapsedTime <= 4.5) {

			direction = glm::vec3(4.5-elapsedTime,0,-(elapsedTime-3.5));
		}
		else if (elapsedTime > 4.5) {
			direction = glm::vec3(-(elapsedTime-4.5), 0,-(5.5 - elapsedTime));
		}
		else {
			direction = glm::vec3(1, 0, 0);
		}
		look = position + direction;
		up = glm::vec3(0, 1, 0);
	}
	else if(elapsedTime < 7.5) {
		// Up movement 
		position += glm::vec3(0, 1, 0) * deltaTime * speed;
		direction = glm::vec3(-1, 0, 0);
		look = position + direction;
		up = glm::vec3(0, 1, 0);
	}
	else if (elapsedTime < 9.8) {
		position += glm::vec3(-1, 0, 0) * deltaTime * speed;
		direction = glm::vec3(-2, 0, 0);
		// Smooth the transition between the forward movement and the curve 
		if (elapsedTime >= 8.8) {
			look = glm::mix(position + direction, glm::vec3(-2.0f, 8.65f, 1.0f), elapsedTime - 8.8f);
		}
		else {
			look = position + direction;
			
		}
		up = glm::vec3(0, 1, 0);
	}
	else if (elapsedTime < 11.0) {
		// Diagonal mvt 
		position += glm::vec3(-1*sqrt(2), 0, -1*sqrt(2)) * deltaTime * speed;
		look = glm::vec3(-2.0f, 8.65f, 1.0f);
		up = glm::vec3(0, 1, 0);
	}
	else if (elapsedTime < 12.2) {
		// Diagonal mvt 
		position += glm::vec3(-sqrt(2), 0, sqrt(2)) * deltaTime * speed;
		look = glm::vec3(-2.0f, 8.65f, 1.0f);
		up = glm::vec3(0, 1, 0);
	}
	else if (elapsedTime < 13.7) {
		// Up 
		position += glm::vec3( 0, 1, 0) * deltaTime * speed;
		direction = glm::vec3(1, 0, 0);
		look = position + direction;
		up = glm::vec3(0, 1, 0);
	}
	else if (elapsedTime < 16) {
		//Forward 
		position += glm::vec3(1, 0,0) * deltaTime * speed;
		direction = glm::vec3(1, 0, 0);
		up = glm::vec3(0, 1, 0);
		// Begin to look at the particles 
		if (elapsedTime > 14.5) {
			look = glm::mix(position + direction, glm::vec3(4.0f, 17.0f, 0.0f), (elapsedTime - 14.5f)/1.5f);
		}
		else {
			look = position + direction;
		}
	}
	else if (elapsedTime < 18) {
		// Small stop to look at the particles 
		position += glm::vec3(0, 0, 0) * deltaTime * speed;
		look = glm::vec3(4.0f, 17.0f, 0.0f);
		up = glm::vec3(0, 1, 0);
	}
	else if (elapsedTime < 25) {
		// De zoom 
		position += glm::vec3(0, 0.1, 1) * deltaTime * speed;
		look = glm::mix(glm::vec3(4.0f, 17.0f, 0.0f), glm::vec3(0.0f, 9.0f, 0.0f), (elapsedTime - 18.0f) / 7.0f);		
		up = glm::vec3(0, 1, 0);
	}
	else if (elapsedTime < 30) {
		// END
		position += glm::vec3(0, 0, 0) * deltaTime * speed;
		look = glm::vec3(0.0f, 9.0f, 0.0f);
		up = glm::vec3(0, 1, 0);
	}
	else {
		// Movement with the arrow keys 

		// Move forward
		if (glfwGetKey(window, GLFW_KEY_UP) == GLFW_PRESS) {
			position += direction * deltaTime * speed;
		}
		// Move backward
		if (glfwGetKey(window, GLFW_KEY_DOWN) == GLFW_PRESS) {
			position -= direction * deltaTime * speed;
		}
		// Strafe right
		if (glfwGetKey(window, GLFW_KEY_RIGHT) == GLFW_PRESS) {
			position += right * deltaTime * speed;
		}
		// Strafe left
		if (glfwGetKey(window, GLFW_KEY_LEFT) == GLFW_PRESS) {
			position -= right * deltaTime * speed;
		}
		look = position + direction;
	}
	

	float FoV = initialFoV;// - 5 * glfwGetMouseWheel(); // Now GLFW 3 requires setting up a callback for this. It's a bit too complicated for this beginner's tutorial, so it's disabled instead.

	// Projection matrix : 45 Field of View, 4:3 ratio, display range : 0.1 unit <-> 100 units
	ProjectionMatrix = glm::perspective(glm::radians(FoV), 4.0f / 3.0f, 0.1f, 100.0f);
	// Camera matrix
	ViewMatrix       = glm::lookAt(
								position,           // Camera is here
								look, // and looks here : at the same position, plus "direction"
								up                  // Head is up (set to 0,-1,0 to look upside-down)
						   );

	// For the next frame, the "last time" will be "now"
	lastTime = currentTime;
}